@@ -166,10 +166,14 @@ class CommandInfo(CommandHelp, ABC):
 
     @classmethod
     def _get_last_option(cls, line: str) -> Optional[str]:
-        parts = reversed([p.strip() for p in lexer.split(line)])
-        for p in parts:
-            if p.startswith("-"):
-                return p
+        try:
+            parts = reversed([p.strip() for p in lexer.split(line)])
+            for p in parts:
+                if p.startswith("-"):
+                    return p
+        except Exception as ex:
+            log.w(f"_get_last_option failed: {ex}")
+
         return None
 
 class FilesSuggestionsCommandInfo(CommandInfo):
@@ -3041,7 +3045,7 @@ Discover Port:    12019
 Authentication:   False
 SSL:              True
 Remote execution: disabled
-Version:          0.10
+Version:          0.12
 
 ================================
 
@@ -3118,7 +3122,7 @@ Usage example:
     **/tmp>** connect alice-arch
     **alice-arch - /tmp> ping**
     [1] PONG from alice-arch (192.168.1.105:12020)  |  time=5.1ms
-    [2] PONG from alice-arch (192.168.1.105:12020)  |  time=0.10ms
+    [2] PONG from alice-arch (192.168.1.105:12020)  |  time=0.12ms
     \.\.\.
     
 2.
