@@ -2,8 +2,9 @@ import os
 from pathlib import Path
 from typing import Optional, Union
 
-if os.name == "nt":
-    # Can't use is_windows() form utils.os due to circular import
+from easyshare.utils.env import is_windows
+
+if is_windows() == "nt":
     import win32api, win32con
 
 
@@ -16,8 +17,12 @@ def is_hidden(p: Union[str, Path]):
         raise TypeError("Path should be str or Path")
 
     if os.name == "nt":
-        attribute = win32api.GetFileAttributes(name)
-        return attribute & (win32con.FILE_ATTRIBUTE_HIDDEN | win32con.FILE_ATTRIBUTE_SYSTEM)
+        try:
+            attribute = win32api.GetFileAttributes(name)
+            return attribute & (win32con.FILE_ATTRIBUTE_HIDDEN | win32con.FILE_ATTRIBUTE_SYSTEM)
+        except:
+            # FIXME
+            return False
     else:
         return name.startswith(".")
 
