@@ -1,7 +1,9 @@
-from ctypes import c_int, CDLL, c_char_p
-from typing import Optional
+from ctypes import c_int, CDLL, c_char_p, CFUNCTYPE
+from typing import Optional, Callable
 
 from easyshare.logging import get_logger
+from easyshare.utils import c
+from easyshare.utils.c import CException
 
 log = get_logger(__name__)
 
@@ -11,75 +13,32 @@ def rl_load():
     global _libreadline
 
     try:
-        import ctypes
-        _libreadline = ctypes.CDLL("libreadline.so")
-        log.d("libreadline loaded with ctypes")
+        _libreadline = c.load_library("libreadline.so")
     except:
         log.w("libreadline can't be accessed with ctypes")
 
 
-# UTIL
-
-def rl_set_int(varname: str, value: int):
+def _call(name: str):
     try:
-        var = c_int.in_dll(_libreadline, varname)
-        log.d(f"SET {varname} was {var} ({var.value})")
-        var.value = value
-        log.d(f"SET {varname} is {var.value}")
-    except Exception as e:
-        log.w(f"SET {varname} failed: {e}")
+        return c.call(_libreadline, name)
+    except CException:
+        pass
 
 
-def rl_get_int(varname: str) -> Optional[int]:
-    try:
-        var = c_int.in_dll(_libreadline, varname)
-        log.d(f"GET {varname} is {var} ({var.value})")
-        return var.value
-    except Exception as e:
-        log.w(f"GET {varname} failed: {e}")
-        return None
+def rl_reset_line_state():
+    _call("rl_reset_line_state")
 
 
-def rl_set_char_p(varname: str, value: bytes):
-    try:
-        var = c_char_p.in_dll(_libreadline, varname)
-        log.d(f"SET {varname} was {var} ({var.value})")
-        var.value = value
-        log.d(f"SET {varname} is {var.value}")
-    except Exception as e:
-        log.w(f"SET {varname} failed: {e}")
-
-
-def rl_get_char_p(varname: str) -> Optional[bytes]:
-    try:
-        var = c_char_p.in_dll(_libreadline, varname)
-        log.d(f"GET {varname} is {var} ({var.value})")
-        return var.value
-    except Exception as e:
-        log.w(f"GET {varname} failed: {e}")
-        return None
-
-def rl_get_char_p_as_str(varname: str) -> Optional[str]:
-    try:
-        var = c_char_p.in_dll(_libreadline, varname)
-        log.d(f"GET {varname} is {var} ({var.value})")
-        return str(var.value, encoding="ascii")
-    except Exception as e:
-        log.w(f"GET {varname} failed: {e}")
-        return None
-
-
-
-# ---- rl_completion_quote_character ---
-"""
-/* Set to any quote character readline thinks it finds before any application
-completion function is called. */
-"""
-
 def rl_get_completion_quote_character() -> int:
-    return rl_get_int("rl_completion_quote_character")
+    """
+    /* Set to any quote character readline thinks it finds before any application
+    completion function is called. */
+    """
+    try:
+        return c.get_int(_libreadline, "rl_completion_quote_character")
+    except CException:
+        return -1
 
-# ---- rl_completer_quote_characters ---
 """
 /* List of characters which can be used to quote a substring of the line.
 Completion occurs on the entire substring, and within the substring
@@ -90,40 +49,94 @@ unless they also appear within this list. */
 _rl_completer_quote_characters = None # keep reference for avoid GC
 
 def rl_set_completer_quote_characters(chars: str):
-    global _rl_completer_quote_characters
-    _rl_completer_quote_characters = chars.encode("ascii")
-    rl_set_char_p("rl_completer_quote_characters", _rl_completer_quote_characters)
+    try:
+        global _rl_completer_quote_characters
+        _rl_completer_quote_characters = c.c_stob(chars)
+        c.set_char_p(_libreadline, "rl_completer_quote_characters", _rl_completer_quote_characters)
+    except:
+        pass
 
-def rl_get_completer_quote_characters():
-    return rl_get_char_p_as_str("rl_completer_quote_characters")
 
-# ---- rl_basic_quote_characters ---
-"""
-/* List of quote characters which cause a word break. */
-"""
+def rl_get_completer_quote_characters() -> str:
+    try:
+        return c.get_char_p(_libreadline, "rl_completer_quote_characters")
+    except:
+        return ""
 
-_rl_basic_quote_characters = None
 
-def rl_set_basic_quote_characters(chars: str):
-    global _rl_basic_quote_characters
-    _rl_basic_quote_characters = chars.encode("ascii")
-    rl_set_char_p("rl_basic_quote_characters", _rl_basic_quote_characters)
-
-def rl_get_basic_quote_characters():
-    return rl_get_char_p_as_str("rl_basic_quote_characters")
-
-# ---- rl_basic_quote_characters ---
-"""
-/* If non-zero, the completion functions don't append any closing quote.
-This is set to 0 by rl_complete_internal and may be changed by an
-application-specific completion function. */
-"""
 
 def rl_set_completion_suppress_quote(value: int):
-    rl_set_int("rl_completion_suppress_quote", value)
+    """
+    /* If non-zero, the completion functions don't append any closing quote.
+    This is set to 0 by rl_complete_internal and may be changed by an
+    application-specific completion function. */
+    """
+    try:
+        c.set_int(_libreadline, "rl_completion_suppress_quote", value)
+    except:
+        pass
 
 
-# -----------------------------------
+
+PROTOTYPE_rl_char_is_quoted_p = CFUNCTYPE(c_int, c_char_p, c_int)
+
+_rl_char_is_quoted_p_wrapper = None # keep reference for avoid GC
+
+def rl_set_char_is_quoted_p(quote_detector: Callable[[str, int], int]):
+    """
+    /* Function to call to decide whether or not a word break character is
+       quoted.  If a character is quoted, it does not break words for the
+       completer. */
+
+    int quote_detector(char * text, int index);
+    """
+
+    global _rl_char_is_quoted_p_wrapper
+
+    def rl_char_is_quoted_p_wrapper(text: bytes, index: int) -> int:
+        log.d(f"rl_char_is_quoted_p_wrapper({text}, {index})")
+        return quote_detector(c.c_btos(text), index)
+
+    _rl_char_is_quoted_p_wrapper = PROTOTYPE_rl_char_is_quoted_p(rl_char_is_quoted_p_wrapper)
+
+    try:
+        c.set_func_ptr(_libreadline, "rl_char_is_quoted_p", _rl_char_is_quoted_p_wrapper)
+    except:
+        pass
+
+
+
+PROTOTYPE_rl_bind_keyseq = CFUNCTYPE(c_int, c_int, c_int)
+_rl_bind_keyseq_wrappers = [] # keep reference for avoid GC
+
+def rl_bind_keyseq(sequence: str, func: Callable[[int, int], int]):
+    """
+    /* Bind the key sequence represented by the string keyseq to the function function,
+    beginning in the current keymap.
+    This makes new keymaps as necessary.
+    The return value is non-zero if keyseq is invalid.   */
+
+    typedef int	  rl_command_func_t(int, int);
+    """
+    wrapper = PROTOTYPE_rl_bind_keyseq(func)
+    _rl_bind_keyseq_wrappers.append(wrapper)
+    return _libreadline.rl_bind_keyseq(sequence, wrapper)
+
+
+PROTOTYPE_rl_bind_key = CFUNCTYPE(c_int, c_int, c_int)
+_rl_bind_key_wrappers = [] # keep reference for avoid GC
+
+def rl_bind_key(key: int, func: Callable[[int, int], int]) -> int:
+    """
+    /* Binds key to function in the currently active keymap.
+    Returns non-zero in the case of an invalid key.  */
+
+    typedef int	  rl_command_func_t(int, int);
+    """
+    wrapper = PROTOTYPE_rl_bind_key(func)
+    _rl_bind_key_wrappers.append(wrapper)
+    return _libreadline.rl_bind_key(key, wrapper)
+
 # from readline.h
 
 """
